@@ -5,15 +5,13 @@ import datetime
 import logging
 import mimetypes
 import rdflib
-import socket;
+import socket
+import urlparse
 from rdflib.namespace import split_uri
 from rdflib import compare
-from urlparse import urlparse
 from xml.sax._exceptions import SAXParseException
 
-import utils
-
-# TODO: find a better way to set socket timeout
+# socket timeout set to 5 seconds globally
 socket.setdefaulttimeout(5)
 
 logger = logging.getLogger(__name__)
@@ -40,19 +38,19 @@ def catchKeyboardInterrupt(func):
         try:
             func(*args, **kwargs)
         except KeyboardInterrupt, _e:
-            print 'KeyboardInterrupt --> %s cancelled' % func
+            print 'KeyboardInterrupt --> Cancelling %s' % func
     return dec
 
 
 def canonalize_uri(uriref):
-    """Returns Uri that is valid or raises Exception"""
+    """Returns Uri that is valid and canonalized or raises Exception"""
     if not uriref:
         raise UriNotValid("uri is None")
 
-    # TODO: if I would implement canonalization following rfc (cut port,
-    # lowercase domain, http and https is equal then I couldn't compare graphs
-    # anymore
+    # TODO: implement canonalization: cut port 80, lowercase domain,
+    # http and https is equal. Problem: graph comparison
 
+    # Workaround for rdflib's handling of BNodes
     if isinstance(uriref, rdflib.BNode):
         return uriref
 
@@ -62,7 +60,7 @@ def canonalize_uri(uriref):
         raise UriNotValid("Not valid: %s" % uriref)
 
     if not hasattr(uriref, "n3"):
-        logger.debug(u"Not an URIRef: %s, fixing that." % uriref)
+        logger.debug(u"Converting %s to URIRef" % uriref)
         uriref = rdflib.URIRef(uriref)
 
     if uriref.startswith('#'):
@@ -73,17 +71,28 @@ def canonalize_uri(uriref):
 
 def hash_to_slash_uri(uri):
     """Converts Hash to Slash uri http://www.w3.org/wiki/HashURI"""
-    # TODO: replace with urlparse
-    assert isinstance(uri, rdflib.URIRef)
-    if '#' in uri:
-        real_uri, hash_value = uri.split("#")
-        assert real_uri
-        return real_uri
+    if not isinstance(uri, rdflib.URIRef):
+        logger.error("Cannot analyse %s (type %s)" % (uri, type(uri)))
+        return uri
+
+    parsed = urlparse.urlparse(uri)
+    uri = urlparse.urlunparse((parsed.scheme, parsed.netloc, parsed.path,
+                         parsed.params, parsed.query,""))
     return uri
 
 
+def convert_to_rdflib_object(v):
+    if hasattr(v, "n3"):
+        return v
+
+    if hasattr(v, "startswith") and v.startswith("http://"):
+        # float has no attribute startswith
+        return rdflib.URIRef(v)
+    else:
+        return rdflib.Literal(v)
+
+
 def reverse_dict(dct):
-    # TODO rdflib.URIRef dict?
     res = {}
     for k,v in dct.iteritems():
         res[v] = k
@@ -108,14 +117,12 @@ def predicate2pyattr(predicate, namespace_short_notation_reverse_dict):
 
 
 def pyattr2predicate(pyattr, namespace_dict):
-    # TODO: build urirefdict instead of unicodedict?
-
     if pyattr.startswith(u"http://"):
         return rdflib.URIRef(pyattr)
 
     splitlist = pyattr.split("_")
 
-    # HACK: are there constrains for namespace prefixes?
+    # TODO: check for namespace prefix limitations
     splitlistlen = len(splitlist)
     if splitlistlen == 1:
         # attribute "homepage" --> check if "" in namespace_dict
@@ -123,8 +130,8 @@ def pyattr2predicate(pyattr, namespace_dict):
         property_name = splitlist[0]
     elif (splitlistlen > 2 and
         u"_".join(splitlist[0:2]) in namespace_dict):
-        # http://www.geonames.org/ontology# defines 'wgs84_pos' --> \
-        # 'wgs84_pos_lat' cannot be solved with approach we took until now
+        # manually handle 'wgs84_pos_lat'
+        # http://www.geonames.org/ontology#
         prefix = u"_".join(splitlist[0:2])
         property_name = u"_".join(splitlist[2:])
         assert prefix, pyattr
@@ -135,24 +142,21 @@ def pyattr2predicate(pyattr, namespace_dict):
 
     assert property_name, pyattr
 
-    # timbl foaf defines foaf as "" --> mbox_sha1sum makes problems here
+    # i.e.: foaf defined as "" --> manually handle 'mbox_sha1sum'
     if "" in namespace_dict:
         if not prefix in namespace_dict:
             logger.error("problem. %s, %s" % (prefix, pyattr))
             return rdflib.URIRef(u"%s%s"
         % (namespace_dict[""], pyattr))
     else:
-        assert namespace_dict[prefix], (u"%s not in namespace_dict. Trying to "
-            "solve %s") % (prefix, pyattr)
+        assert namespace_dict[prefix], (u"%s not in namespace_dict") % prefix
 
-    return rdflib.URIRef(u"%s%s"
-        % (namespace_dict[prefix], property_name))
+    return rdflib.URIRef(u"%s%s" % (namespace_dict[prefix], property_name))
 
 
 from backends import *
 from models import *
 
-######### Resource #########
 
 class ResourceManager(Manager):
 
@@ -161,31 +165,45 @@ class ResourceManager(Manager):
 
     def create(self, uri, origin, **kwargs):
         assert isinstance(origin, Origin), "Origin instance required"
-        #assert origin.processed, ("Origin has to be processed before creating "
-        #    "more Resource objects: origin.GET()")
+        assert origin.processed, ("Origin has to be processed "
+            "before creating more Resource objects: origin.GET()")
 
+        # Absolutize relative URIs
         if uri.startswith("#"):
             assert not isinstance(uri, rdflib.BNode), \
-                "bnode should not start with #"
+                "BNode not allowed to start with #"
             uri = rdflib.URIRef(origin.uri + uri)
 
         uri = canonalize_uri(uri)
+
         pk = self.get_pk(origin_uri=origin.uri, uri=uri)
+        return super(ResourceManager, self).create(
+            pk=pk, _uri=uri, _origin=origin, **kwargs)
 
-        obj = super(ResourceManager, self).create(
-            pk=pk, _uri=uri, _origin=origin, **kwargs
-        )
+    def get_authoritative_resource(self, uri,
+                                   create_nonexistent_origin=True):
+        """Tries to return the Resource object from the original origin"""
+        uri = canonalize_uri(uri)
+        origin_uri = hash_to_slash_uri(uri)
 
-        # move to tests
-        if isinstance(uri, rdflib.BNode):
-            assert isinstance(obj._uri, rdflib.BNode)
+        authoritative_origin = Origin.objects.filter(uri=origin_uri)
+        authoritative_origin_list = list(authoritative_origin)
+        if len(authoritative_origin_list) == 1:
+            origin = authoritative_origin_list[0]
+        else:
+            if create_nonexistent_origin:
+                origin = Origin.objects.create(uri=origin_uri)
+            else:
+                raise self.model.DoesNotExist("No authoritative "
+                "Resource found for %s" %uri)
 
-        assert not hasattr(obj, "_has_changes")
+        # TODO: make this configurable
+        origin.GET(only_follow_uris=[])
 
-        return obj
+        authoritative_resource = self.get(uri, origin)
+        return authoritative_resource
 
-
-    def get(self, uri, origin=None, return_authoritative_resource=True):
+    def get(self, uri, origin=None):
         """If the authoratative Origin to the Resource does not exist and no
         origin is given then DoesNotExist is returned. Assumption is
         to only trust validated sources.
@@ -208,38 +226,19 @@ class ResourceManager(Manager):
                 # return only match
                 return filter_result[0]
             else:
-                if return_authoritative_resource:
-                    # try to return best match
-                    authoritative_resource = None
-                    for resource in filter_result:
-                        if uri.startswith(resource._origin.uri):
-                            authoritative_resource = resource
-                            break
-                    if authoritative_resource:
-                        return authoritative_resource
-                    else:
-                        raise self.model.DoesNotExist("No authoritative "
-                            "Resource found for %s" %uri)
-                else:
-                    raise self.model.DoesNotExist("Please pass the exact "
-                        "Origin. The Resource you are looking for is "
-                        "provided by the Origins: %s"
-                        % ", ".join([unicode(r._origin.uri)
-                                     for r in filter_result]))
+                raise self.model.DoesNotExist("Please pass the exact "
+                    "Origin. The Resource you are looking for is "
+                    "provided by the Origins: %s"
+                    % ", ".join([unicode(r._origin.uri)
+                                 for r in filter_result]))
 
         assert isinstance(origin, Origin), origin
         pk = self.get_pk(origin_uri=origin.uri, uri=uri)
         return super(ResourceManager, self).get(pk=pk)
         assert 0, "implement!"
 
-    def get_or_create(self, uri, origin=None, auto_origin=False):
+    def get_or_create(self, uri, origin=None):
         uri = canonalize_uri(uri)
-        if auto_origin:
-            assert not origin, "Either origin or auto_origin, not both"
-            origin_uri = hash_to_slash_uri(uri)
-            origin, _origin_created = Origin.objects\
-                .get_or_create(uri=origin_uri)
-
         try:
             return self.get(uri=uri, origin=origin), False
         except self.model.DoesNotExist:
@@ -253,24 +252,22 @@ class Resource(Model):
     objects = ResourceManager()
 
     def __unicode__(self):
-        str = u"%s" % self._uri
-        if hasattr(self, 'foaf_name'):
-            str += u' "%s"' % unicode(self.foaf_name)
-            if len(self.foaf_name) > 1: str += u",..."
-        if hasattr(self, "_origin") and isinstance(self._origin, Origin):
-            str += u" [%r]" % self._origin.uri.encode("utf8")
-        return str
+        str = [u"%r" % self._uri.encode('utf8')]
+        if hasattr(self, "_origin"):
+            assert isinstance(self._origin, Origin)
+            if self.is_authoritative_resource():
+                str.append(u"*authoritative*")
+            else:
+                str.append(u"[%r]" % self._origin.uri.encode("utf8"))
+        return " ".join(str)
 
-    def get_attributes(self):
-        dct = copy.copy(self.__dict__)
-        for attr in [u"pk",
-            u"_origin",
-            u"_uri",
-            u"_reverse",
-            u"_has_changes"]:
-            if hasattr(self, attr):
-                dct.pop(attr)
-        return dct
+    def is_authoritative_resource(self):
+        if not isinstance(self._uri, rdflib.URIRef):
+            # BNodes don't have an URI
+            logger.warning("Cannot compare %s" % type(self._uri))
+            return False
+        if hash_to_slash_uri(self._uri) == str(self._origin.uri):
+            return True
 
     def _add_property(self, predicate, object,
                       namespace_short_notation_reverse_dict):
@@ -286,7 +283,7 @@ class Resource(Model):
             is_resource = True
         elif isinstance(object, rdflib.URIRef):
             if not isinstance(object, rdflib.Literal):
-                o = urlparse(object)
+                o = urlparse.urlparse(object)
                 if o.scheme == "http":
                     is_resource = True
                 else:
@@ -353,6 +350,8 @@ class Resource(Model):
                         # If object is referenced in attribute, "de-reference"
                         yield((self._uri, property, v._uri))
                     else:
+                        if not hasattr(v, "n3"):
+                            v = convert_to_rdflib_object(v)
                         yield((self._uri, property, v))
             else:
                 v = values
@@ -360,48 +359,38 @@ class Resource(Model):
                     # If object is referenced in attribute, "de-reference"
                     yield((self._uri, property, v._uri))
                 else:
+                    if not hasattr(v, "n3"):
+                        v = convert_to_rdflib_object(v)
                     yield((self._uri, property, v))
 
-
     def __setattr__(self, key, value):
-        # TODO: is there a better way to validate attributes?
+        if key == "_has_changes":
+            Model.__setattr__(self, key, value)
+            return
+
         if key in self._meta.fields:
             field = self._meta.fields.get(key)
             if field and value:
                 value = field.to_python(value)
-        elif not key.startswith("_") and not key == "pk": # TODO: rename to_pk?
-            # rdf properties here
-            self._has_changes = True
+        elif not key.startswith("_") and not key == "pk":
+            # Assumption: rdf attributes do not start with "_"
+            pass # TODO: validate
         Model.__setattr__(self, key, value)
+        self._has_changes = True
 
     def delete(self):
-        if hasattr(self, "_has_changes"):
-            assert not self._has_changes
-        assert hasattr(self, "pk") and self.pk is not None
-        # TODO: use Collector objects as django does
-        self.__class__.objects._storage.__delitem__(self.pk)
+        if hasattr(self, "pk") and self.pk is not None:
+            self.__class__.objects._storage.__delitem__(self.pk)
 
     def save(self):
-        # TODO: introduce "clean" to make sure only valid properties modified
+        # TODO Move to models.py and introduce "clean" to validate properties
         created = not self.pk
         if created:
-            assert 0, ("Please use Resource.objects.create() to create Resource"
-                       "objects!")
-            #assert not self.__class__.objects.get(self.uri, self.origin)
-            #self = self.__class__.objects.create(**self.get_attributes())
-
-        # TODO: write tests for the following lines
+            assert 0, ("Please use Resource.objects.create() to create "
+                       "Resource objects!")
         assert self in Resource.objects.filter(_origin=self._origin)
-
-        if self in Resource.objects.filter(_origin=self._origin,
-                _has_changes=True):
-            assert len(list(Resource.objects.filter(_origin=self._origin,
-                _has_changes=True))) == 1
-        else:
-            logging.info("New resource object was created")
-
-        #values = dict((name, getattr(self, name)) for name in \
-        #        self._meta.fields.iterkeys())
+        #values = dict((name, getattr(self, name)) for name in
+        # self._meta.fields.iterkeys())
         self.update() #**values)
 
     def update(self):
@@ -409,14 +398,10 @@ class Resource(Model):
         self._has_changes = False
 
 
-######### Origin #########
-
 class OriginManager(Manager):
 
     def create_hook(self, uri):
-        assert not '#' in uri, ("HashURI not allowed as Origin: %s. Maybe "
-            "you are looking for "
-            "Resource.objects.get_or_create(...,auto_origin=True)?" % uri)
+        assert not '#' in uri, ("HashURI not allowed as Origin: %s" % uri)
 
     def create(self, uri, BACKEND=None):
         uri = canonalize_uri(uri)
@@ -468,14 +453,14 @@ class Origin(Model):
         self.processed = False
 
     def __unicode__(self):
-        str = u"%s" % unicode(self.uri)
-        str += " %s" % self.backend.__class__.__name__
+        str = [unicode(self.uri)]
+        str.append(self.backend.__class__.__name__)
         if hasattr(self, 'errors'):
             for error in self.errors:
-                str += u" %s" % error
+                str.append(unicode(error))
         if self.processed:
-            str += u" Processed"
-        return str
+            str.append(u"Processed")
+        return u" ".join(str)
 
     def GET(self,
             GRAPH_SIZE_LIMIT=30000,
@@ -488,6 +473,11 @@ class Origin(Model):
         if not self.uri:
             raise Exception("Please provide URI first")
 
+        if skip_urls is not None and str(self.uri) in skip_urls:
+            self.add_error("Skipped")
+            self.processed = True
+            return
+
         logger.info(u"GET %s..." % self.uri)
 
         if self.has_unsaved_changes():
@@ -498,11 +488,6 @@ class Origin(Model):
                 logger.warning("There were Resource objects created before "
                                "processing the resource's origin.")
 
-        if skip_urls is not None and str(self.uri) in skip_urls:
-            self.add_error("Skipped")
-            self.processed = True
-            return
-
         self.stats['last_processed'] = datetime.datetime.now()
 
         try:
@@ -522,20 +507,22 @@ class Origin(Model):
             if raise_errors: raise e
             else: return
 
-        if not data:
-            self.processed = True
-            return
-
-        assert self.backend.format, "format is needed later"
-
         graph = rdflib.graph.ConjunctiveGraph(identifier=self.uri)
 
+
         try:
-            # Important: Do not pass data=data without publicID=uri because
-            # relative URIs (#deri) won't be an absolute uri in that case!
-            assert data
-            graph.parse(data=data, publicID=self.uri,
-                        format=self.backend.format)
+            if data:
+                # Important: Do not pass data=data without publicID=uri because
+                # relative URIs (#deri) won't be an absolute uri in that case!
+                publicID = self.uri
+                graph.parse(data=data,
+                            publicID=publicID,
+                            format=self.backend.format)
+
+                # normal rdflib.compare does not work correctly with
+                # ConjunctiveGraph, unless there is only one graph within that
+                assert len(list(graph.contexts())) == 1
+
         except SAXParseException as e:
             self.add_error("SAXParseException")
             logger.error("SAXParseException: %s" % self)
@@ -554,39 +541,36 @@ class Origin(Model):
             else: return
 
         self.processed = True
-        if not graph:
-            logger.warning("%s did not return any information" % self.uri)
-            return
 
-        if hasattr(self, "errors"): delattr(self, "errors")
+        if hasattr(self, "errors"):
+            delattr(self, "errors")
 
         g_length = len(graph)
-        if g_length > 5000:
-            logger.warning("len(graph) == %s" % g_length)
-            if g_length > GRAPH_SIZE_LIMIT:
-                logger.error("Maximum graph size is set to %s. The aquired "
-                             "graph exceeds that! Pass "
-                             "GRAPH_SIZE_LIMIT to set it differently."
-                             % GRAPH_SIZE_LIMIT);
-                return
-
-        # normal rdflib.compare does not work correctly with
-        # conjunctiveGraph, unless there is only one graph within that
-        assert len(list(graph.contexts())) == 1
+        if g_length > GRAPH_SIZE_LIMIT:
+            logger.error("Maximum graph size exceeded. Thr graph is %s "
+                         "triples big. Limit is set to %s. The aquired "
+                         "graph exceeds that! Pass GRAPH_SIZE_LIMIT to set it "
+                         "differently." % (g_length, GRAPH_SIZE_LIMIT))
+            return
 
         if hasattr(self, "_graph"):
             # we already assured that there are no unsaved_changes
-            # --> graph() == _graph
+            # --> get_graph() == _graph
             logger.info(u"Already crawled: %s. Comparing graphs..." % self.uri)
 
-            if not compare.to_isomorphic(self._graph) == \
-                   compare.to_isomorphic(graph):
+            if compare.to_isomorphic(self._graph) == \
+               compare.to_isomorphic(graph):
+                return
+            else:
                 logging.warning("GET retrieved updates for %s!" % self.uri)
-                utils.my_graph_diff(self._graph, graph)
+                try:
+                    import utils
+                    utils.my_graph_diff(self._graph, graph)
+                except ImportError:
+                    pass
 
                 for resource in self.get_resources():
                     resource.delete()
-
                 delattr(self, "handled")
 
         if hasattr(self, "handled"):
@@ -594,90 +578,33 @@ class Origin(Model):
 
         self._graph = graph
 
-        if not list(self.get_resources()):
-            logger.debug("Resources exist but no _graph --> Resources were "
-                         "created locally")
-
-        def create_origin(o, caused_by=None, # should be Origin object
-                                process_now=False):
-            if not type(o) == rdflib.URIRef:
-                return
-
-            uri = hash_to_slash_uri(o)
-            origin, created = Origin.objects.get_or_create(uri=uri)
-            if created:
-                setattr(origin, '_created_by', caused_by)
-            if process_now:
-                logger.info("Interrupting to load %s because we need to "
-                        "process owl:imports %s first" % (caused_by.uri,
-                                                          origin.uri))
-                origin.GET()
-
-                
-        namespace_short_notation_reverse_dict = reverse_dict(dict(graph\
-            .namespace_manager.namespaces()))
-
-        # TODO: pass only_follow_uris = [] to skip creating origins? --> find
-        # a better way
-        if only_follow_uris is not None:
-            only_follow_uris = [rdflib.URIRef(u) if not\
-                isinstance(u, rdflib.URIRef) else u for u in only_follow_uris]
-
         start_time = datetime.datetime.now()
 
-        for subject, predicate, obj_ect in graph:
-            assert hasattr(subject, "n3")
-            #subject = canonalize_uri(subject)
-
-            assert predicate.encode('utf8')
-
-            if handle_owl_imports:
-                if predicate == rdflib.OWL.imports:
-                    create_origin(obj_ect, caused_by=self, process_now=True)
-
-            if only_follow_uris is not None:
-                if predicate in only_follow_uris:
-                    create_origin(obj_ect, caused_by=self)
-            else:
-                # follow every URIRef! this could take a long time!
-                create_origin(obj_ect, caused_by=self)
-
-            resource, _created = Resource.objects.get_or_create(uri=subject,
-                                                                origin=self)
-            resource._add_property(predicate, obj_ect,
-                                   namespace_short_notation_reverse_dict)
+        graph_handler = GraphHandler(
+            only_follow_uris=only_follow_uris,
+            handle_owl_imports=handle_owl_imports,
+            origin=self)
+        graph_handler.populate_resources(graph=graph)
 
         self.stats['graph_processing_time'] = datetime.datetime.now() - start_time
 
-        for resource in self.get_resources():
-            resource._has_changes = False
-
-        assert compare.to_isomorphic(graph) == \
-               compare.to_isomorphic(self.graph()), \
-               utils.my_graph_diff(graph, self.graph())
-
         self.handled = True
 
-
-    def get_resources(self):
-        return Resource.objects.filter(_origin=self)
-
-    def graph(self):
-        """Processes every Resource and Property related to 'self' and
-        creates rdflib.ConjunctiveGraph because rdflib.Graph does not allow
-        parsing plugins
-        """
-        # TODO: rdflib.graph() ?
+    def get_graph(self):
+        """Processes every Resource and Property related to 'self'"""
+        #rdflib.ConjunctiveGraph because rdflib.Graph does not allow parsing plugins
         graph = rdflib.graph.ConjunctiveGraph(identifier=self.uri)
 
         if not hasattr(self, '_graph'):
-            if len(self.errors) == 0:
-                self.GET(raise_errors=False) # TODO: test for recursion?
-            else:
-                logging.error("Origin %s has Errors --> can't process .graph()"
+            if hasattr(self, 'errors') and len(self.errors) != 0:
+                logging.error("Origin %s has Errors --> can't process "
+                              ".get_graph()"
                     % self.uri)
                 return graph
 
+            #self.GET(raise_errors=False) # TODO: test for recursion?
+
+
         # TODO: find a better way to do this
         # Problems:
         #  1) namespace bindings are not really necessary to validate
@@ -703,15 +630,15 @@ class Origin(Model):
                 graph.add(triple)
         return graph
 
-    def has_unsaved_changes(self):
-        # resource objects exist although not processed yet
-        if not self.processed and len(list(self.get_resources())):
-            return True
+    def get_resources(self):
+        return Resource.objects.filter(_origin=self)
 
+    def has_unsaved_changes(self):
         # objects with changed attributes exist
         if any(resource._has_changes
                 for resource in self.get_resources()\
-                    if hasattr(resource, '_has_changes')):
+                    if (hasattr(resource, '_has_changes')
+                        and resource._has_changes == True)):
             return True
 
         return False
@@ -726,36 +653,18 @@ class Origin(Model):
             logging.error("Nothing to PUT for %s!" % self.uri)
             return
 
-        graph = self.graph()
+        graph = self.get_graph()
         data = graph.serialize(format=self.backend.format)
 
+        # TODO: synchronize if remote resource is still up to date?
         self.backend.PUT(data=data)
+
+        for resource in Resource.objects.filter(_has_changes=True):
+            resource._has_changes = False
+
+        assert not self.has_unsaved_changes(), "something went wrong"
         # TODO return "OK"
 
-    def get_statistics(self):
-        def triples_per_second(triples, time): # TODO make this more accurate
-            total_seconds = (time.microseconds+(time.seconds+\
-                                      time.days*24*3600)*10**6)//10**6
-            return triples / total_seconds if total_seconds > 0 else None
-
-        if hasattr(self, '_graph'):
-            triples = len(self._graph)
-            tps = triples_per_second(triples,
-                                     self.stats['graph_processing_time'])
-            if tps:
-                logger.info(
-                    "Crawled %s: '%s' triples in '%s' seconds --> '%s' "
-                    "triples/second"
-                    % (self.uri, triples,
-                       self.stats['graph_processing_time'], tps))
-            else:
-                logger.info("Crawled %s in '%s' seconds"
-                % (self.uri, self.stats['graph_processing_time']))
-            pass
-        else:
-            print "no statistics recorded"
-
-
 
 def check_shortcut_consistency():
     """Checks every known Origin for inconsistent namespacemappings"""
@@ -768,3 +677,57 @@ def check_shortcut_consistency():
                     assert global_namespace_dict[k] == v
                 else:
                     global_namespace_dict[k] = v
+
+
+class GraphHandler(object):
+    def __init__(self, origin, only_follow_uris, handle_owl_imports):
+        self.origin=origin
+
+        self.handle_owl_imports=handle_owl_imports
+        if only_follow_uris is not None:
+            only_follow_uris = [rdflib.URIRef(u) if not\
+                isinstance(u, rdflib.URIRef) else u for u in only_follow_uris]
+        self.only_follow_uris=only_follow_uris
+
+    def populate_resources(self, graph):
+        namespace_short_notation_reverse_dict = reverse_dict(dict(graph\
+            .namespace_manager.namespaces()))
+
+        for subject, predicate, obj_ect in graph:
+            assert hasattr(subject, "n3")
+            #subject = canonalize_uri(subject)
+
+            #if isinstance(subject, rdflib.BNode):
+            #    assert 0, "subject is BNode: %s %s %s" % (subject, predicate, obj_ect)
+            #if isinstance(predicate, rdflib.BNode):
+            #    assert 0, "predicate is BNode: %s %s %s" % (subject, predicate, obj_ect)
+            #if isinstance(obj_ect, rdflib.BNode):
+            #    assert 0, "object is BNode: %s %s %s" % (subject, predicate, obj_ect)
+
+            # workaround for rdflib's unicode problems
+            assert predicate.encode('utf8')
+
+            if self.handle_owl_imports:
+                if (predicate == rdflib.OWL.imports
+                    and type(obj_ect) == rdflib.URIRef):
+
+                    uri = hash_to_slash_uri(obj_ect)
+                    origin, created = Origin.objects.get_or_create(uri=uri)
+
+                    logger.info("Interrupting to process owl:imports %s"
+                                "first" % (origin.uri))
+                    origin.GET()
+
+            if ((self.only_follow_uris is not None
+                 and predicate in self.only_follow_uris)
+                or self.only_follow_uris is None):
+                if type(obj_ect) == rdflib.URIRef:
+                    Origin.objects.get_or_create(uri=hash_to_slash_uri(obj_ect))
+
+            resource, _created = Resource.objects.get_or_create(uri=subject,
+                                                        origin=self.origin)
+            resource._add_property(predicate, obj_ect,
+                                   namespace_short_notation_reverse_dict)
+
+        for resource in self.origin.get_resources():
+            resource._has_changes = False
